/*
 *
 *  Copyright (c) 2010 Zagaia (INdT - Instituto Nokia de Tecnologia/
 *       FUCAPI - Fundação Centro de Análise, Pesquisa e Inovação Tecnológica)
 *
 *  This file is part of TweeGo.
 *
 *  TweeGo is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  TweeGo is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with TweeGo. If not, see <http://www.gnu.org/licenses/>
 *
 */

#ifndef TWITTERBUTTON_H
#define TWITTERBUTTON_H

#include <QtCore/QObject>
#include <QtCore/QRectF>
#include <QtCore/QString>

#include <QtGui/QStyleOptionGraphicsItem>
#include <QtGui/QGraphicsSceneMouseEvent>
#include <QtGui/QPixmap>
#include <QtGui/QPainter>
#include <QtGui/QPainterPath>

#include "twitterapplicationitem.h"

/*! \class TwitterButton
 * \brief TwitterButton class
 * Handles all operations related to buttons.
 */
class TwitterButton : public TwitterApplicationItem
{
    Q_OBJECT

public:
    enum ButtonType { Update, Expose, Logout, Generic, Retweet, Reply, Favorite, NoFavorite, GoUrl, Delete, Back, Tab, More };

    TwitterButton(ButtonType buttonType, QString buttonLabel, QGraphicsWidget *parent);
    QPainterPath shape() const;
    QRectF boundingRect() const;
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *);
    void enable(bool);
    void setTypeButton(int);
    void setLabel(QString);
    void setUrl(QString);

signals:
    void pressed();
    void released();
    void released(QString);

protected:
    virtual void mousePressEvent(QGraphicsSceneMouseEvent *);
    virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent *);
    virtual void mouseMoveEvent(QGraphicsSceneMouseEvent *event);

private:
    QPen _pen;
    QPixmap _buttonImage;
    QPixmap _buttonSelectedImage;
    QString _buttonLabel;
    int _buttonType;
    QString _url;
    bool m_isPressed;
};	

#endif // TWITTERBUTTON_H
