/*
 *
 *  Copyright (c) 2010 Zagaia (INdT - Instituto Nokia de Tecnologia/
 *       FUCAPI - Fundação Centro de Análise, Pesquisa e Inovação Tecnológica)
 *
 *  This file is part of TweeGo.
 *
 *  TweeGo is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  TweeGo is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with TweeGo. If not, see <http://www.gnu.org/licenses/>
 *
 */

#ifndef TWITPIC_H
#define TWITPIC_H

#include <QObject>
#include <QDebug>
#include <QString>
#include <QNetworkAccessManager>
#include <QNetworkProxy>
#include <QList>
#include <QStringList>
#include <QListIterator>
#include <QRegExp>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QImage>
#include <QtGui/QPainter>
#include <QWebFrame>
#include <QGraphicsSceneMouseEvent>
#include <QFileDialog>
#include <QMessageBox>

#ifdef LIBLOCATION
    #include <QMaemo5InformationBox>
#endif // LIBLOCATION

#include "applicationItems/twitterapplicationitem.h"

/*!
 * \class TwitPic
 *
 * \brief TwitPic class handles all operations related to TwitPic.
 */
class TwitPic: public TwitterApplicationItem
{
    Q_OBJECT

    public:
        TwitPic(QGraphicsWidget *parent = 0);
        ~TwitPic();
        void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *);
        void download(QString Url);

    signals:
        void errorMessage( const QString &message );
        void urlDownloaded(QString imageUrl);
        void animExit(void);

    protected:
        virtual void mousePressEvent(QGraphicsSceneMouseEvent *);
        virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent *);

    private slots:
        void handleNetworkRequest(QNetworkReply *reply);

    private:
        void parseHtml(QString html);
        void downloadImage(QString imageUrl);

    private:
        QNetworkAccessManager *m_manager;
        QImage m_image;
        QImage m_imageScaled;
        bool m_isImage;
        bool m_paintImage;
};

#endif // TWITPIC_H
